@@ -1,3 +1,23 @@
+/* $BEGIN_LICENSE
+
+This file is part of Minitube.
+Copyright 2009, Flavio Tordini <flavio.tordini@gmail.com>
+
+Minitube is free software: you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
+
+Minitube is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with Minitube.  If not, see <http://www.gnu.org/licenses/>.
+
+$END_LICENSE */
+
 #include "ytstandardfeed.h"
 #include <QtXml>
 #include "networkaccess.h"
@@ -23,8 +43,11 @@ void YTStandardFeed::loadVideos(int max, int skip) {
     QUrl url(s);
     url.addQueryItem("v", "2");
 
-    if (feedId != "most_shared")
-        url.addQueryItem("time", "today");
+    if (feedId != "most_shared" && feedId != "on_the_web") {
+        QString t = time;
+        if (t.isEmpty()) t = "today";
+        url.addQueryItem("time", t);
+    }
 
     url.addQueryItem("max-results", QString::number(max));
     url.addQueryItem("start-index", QString::number(skip));
